import logging 


class Log:

    def __init__(
        self, 
        name: str
    ):
        self.log = logging.getLogger(name)
        self.log.propagate = True
        self.formatter = logging.Formatter("%(asctime)s | [%(levelname)s] | %(message)s",
                              "%Y-%m-%d %H:%M:%S")
        self.levels = {
            "DEBUG" : logging.DEBUG,
            "INFO" : logging.INFO,
            "WARNING" : logging.WARNING,
            "ERROR" : logging.ERROR,
            "CRITICAL" : logging.CRITICAL
        }
    
    def stream_handler(
        self,
        level: str
    ):
        if len(self.log.handlers) > 0:
            return self.log # Logger already exists
        else:
            """
            level :
            > "DEBUG" : logging.DEBUG , 
            > "INFO" : logging.INFO , 
            > "WARNING" : logging.WARNING , 
            > "ERROR" : logging.ERROR , 
            > "CRITICAL" : logging.CRITICAL , 
            """
            self.log.setLevel(self.levels[level])
            streamHandler = logging.StreamHandler()
            streamHandler.setFormatter(self.formatter)
            self.log.addHandler(streamHandler)
            return self.log




import pandas as pd
from typing import (
    List,
    Dict,
    Optional
)
from dataclasses import dataclass
from vdutils.data import _get_folder_names
from vdutils.library import Log
from vdutils.convaddr import ConvAddr


@dataclass
class BjdObject():

    def __init__(
        self,
        bjd_cd: str,
        full_bjd_nm: str,
        sido_list: List[str],
        sgg_list: List[str],
        emd_list: List[str],
        ri_list: List[str],
        multiple_word_sgg_list: List[str],
        base_dt: str
    ):
        self.bjd_cd: str = bjd_cd
        self.full_bjd_nm: str = full_bjd_nm
        self.typ: str = None
        self.sido: Optional[bool] = None
        self.sgg: Optional[bool] = None
        self.emd: Optional[bool] = None
        self.ri: Optional[bool] = None
        self.sido_nm: Optional[str] = None
        self.sgg_nm: Optional[str] = None
        self.emd_nm: Optional[str] = None
        self.ri_nm: Optional[str] = None
        self.sido_cd: Optional[str] = None
        self.sgg_cd: Optional[str] = None
        self.emd_cd: Optional[str] = None
        self.ri_cd: Optional[str] = None
        self.bjd_nm: str = None
        self.base_dt: str = base_dt
        self.base_dt_print: str = f"{self.base_dt[:4]}-{self.base_dt[4:6]}-{self.base_dt[6:8]}"
        self.sido_list: List[str] = sido_list
        self.sgg_list: List[str] = sgg_list
        self.emd_list: List[str] = emd_list
        self.ri_list: List[str] = ri_list
        self.multiple_word_sgg_list: List[str] = multiple_word_sgg_list
        self._prepare()


    def _get_bjd_typ(
        self,
        bjd_nm
    ):
        for bjd_typ_nm, bjd_typ_group in [
            ("sido", self.sido_list),
            ("sgg", self.sgg_list),
            ("emd", self.emd_list),
            ("ri", self.ri_list)
        ]:
            if bjd_nm in bjd_typ_group:
                return bjd_typ_nm
        raise ValueError(f"{bjd_nm}: Invalid input")


    def _get_typ(self):
        if self.ri_nm is not None: return "ri"
        elif self.emd_nm is not None: return "emd"
        elif self.sgg_nm is not None: return "sgg"
        elif self.sido_nm is not None: return "sido"
        else: raise ValueError()


    def _get_bjd_nm(self):
        if self.typ == "sido": return self.sido_nm
        elif self.typ == "sgg": return self.sgg_nm
        elif self.typ == "emd": return self.emd_nm
        elif self.typ == "ri": return self.ri_nm
        else: raise ValueError()


    def _get_bjd_cd(self):
        if self.sido_nm is not None:
            if self.sido_nm == "세종특별자치시": self.sido_cd = self.bjd_cd[:5] + ("0" * 5)
            else: self.sido_cd = self.bjd_cd[:2] + ("0" * 8)
        if self.sgg_nm is not None:
            self.sgg_cd = self.bjd_cd[:5] + ("0" * 5)
        if self.emd_nm is not None:
            self.emd_cd = self.bjd_cd[:8] + ("0" * 2)
        if self.ri_nm is not None:
            self.ri_cd = self.bjd_cd


    def _get_is_existed_bjd(self):
        if self.sido_nm is not None:
            self.sido = True
        if self.sgg_nm is not None:
            self.sgg = True
        if self.emd_nm is not None:
            self.emd = True
        if self.ri_nm is not None:
            self.ri = True


    def _split_full_bjd_nm(self):
        for multiple_sgg_nm in self.multiple_word_sgg_list:
            if multiple_sgg_nm in self.full_bjd_nm:
                return [multiple_sgg_nm] + self.full_bjd_nm.replace(multiple_sgg_nm, "").split()
        return self.full_bjd_nm.split()


    def _prepare(self):
        bjd_nm_list = self._split_full_bjd_nm()
        for bjd_nm in bjd_nm_list:
            bjd_typ = self._get_bjd_typ(bjd_nm)
            if bjd_typ == "sido": self.sido_nm = bjd_nm
            elif bjd_typ == "sgg": self.sgg_nm = bjd_nm
            elif bjd_typ == "emd": self.emd_nm = bjd_nm
            elif bjd_typ == "ri": self.ri_nm = bjd_nm
            else: raise ValueError(f"{bjd_typ}: Invalid input")
        self.typ = self._get_typ()
        self.bjd_nm = self._get_bjd_nm()
        self._get_bjd_cd()
        self._get_is_existed_bjd()


    def _print(self):
        print(f"bjd_cd: {self.bjd_cd}")
        print(f"bjd_nm: {self.bjd_nm}")
        print(f"full_bjd_nm: {self.full_bjd_nm}")
        print(f"typ: {self.typ}")
        print(f"sido: {self.sido}")
        print(f"sgg: {self.sgg}")
        print(f"emd: {self.emd}")
        print(f"ri: {self.ri}")
        print(f"sido_nm: {self.sido_nm}")
        print(f"sgg_nm: {self.sgg_nm}")
        print(f"emd_nm: {self.emd_nm}")
        print(f"ri_nm: {self.ri_nm}")
        print(f"sido_cd: {self.sido_cd}")
        print(f"sgg_cd: {self.sgg_cd}")
        print(f"emd_cd: {self.emd_cd}")
        print(f"ri_cd: {self.ri_cd}")
        print(f"base_dt: {self.base_dt_print}")


@dataclass
class BjdConnector():

    bjd_top_relation = {
        "sido": None,
        "sgg": "sido",
        "emd": "sgg",
        "ri": "emd"
    }


    def __init__(
        self,
        bjd_cd: str,
        full_bjd_nm: str,
        sido_list: List[str],
        sgg_list: List[str],
        emd_list: List[str],
        ri_list: List[str],
        multiple_word_sgg_list: List[str],
        base_dt: str,
    ):
        self.typ: str = None
        self.bjd_cd: str = bjd_cd
        self.bjd_nm: str = None
        self.full_bjd_nm: str = full_bjd_nm
        self.metadata: BjdObject() = None
        self.top_bjd_typ: Optional[str] = None
        self.top_bjd_cd: List[str] = []
        self.top_bjd_nm: List[str] = []
        self.top_bjd: List[BjdConnector()] = []
        self.bottom_bjd_cd: List[str] = []
        self.bottom_bjd_nm: List[str] = []
        self.bottom_bjd: List[BjdConnector()] = []
        self.is_smallest: bool = None
        self.base_dt: str = base_dt
        self.base_dt_print: str = f"{self.base_dt[:4]}-{self.base_dt[4:6]}-{self.base_dt[6:8]}"
        self.sido_list: List[str] = sido_list
        self.sgg_list: List[str] = sgg_list
        self.emd_list: List[str] = emd_list
        self.ri_list: List[str] = ri_list
        self.multiple_word_sgg_list: List[str] = multiple_word_sgg_list
        self._update_metadata()
        self._update_top_bjd()


    def _update_metadata(self):
        bjd_object = BjdObject(
            bjd_cd=self.bjd_cd,
            full_bjd_nm=self.full_bjd_nm,
            sido_list=self.sido_list,
            sgg_list=self.sgg_list,
            emd_list=self.emd_list,
            ri_list=self.ri_list,
            multiple_word_sgg_list=self.multiple_word_sgg_list,
            base_dt=self.base_dt
        )
        self.metadata = bjd_object
        self.typ = bjd_object.typ
        self.bjd_nm = bjd_object.bjd_nm


    def _find_top_relation(
        self,
        typ,
        metadata
    ):
        if typ == "sido":
            return None
        top_typ = self.bjd_top_relation.get(typ)
        if getattr(metadata, f"{top_typ}") is True:
            return top_typ
        return self._find_top_relation(
            typ=top_typ,
            metadata=metadata
        ) # 상위 법정동이 없을 상위 법정동의 상위를 탐색


    def _update_top_bjd(self):
        typ = self.typ
        metadata = self.metadata
        self.top_bjd_typ = self._find_top_relation(
            typ=typ,
            metadata=metadata
        )
        if self.top_bjd_typ is not None:
            if self.top_bjd_typ == "sido":
                if self.metadata.sido_cd is not None:
                    self.top_bjd_nm.append(self.metadata.sido_nm)
                    self.top_bjd_cd.append(self.metadata.sido_cd)
                else:
                    print(self.metadata._print())
            if self.top_bjd_typ == "sgg":
                if self.metadata.sgg_cd is not None:
                    self.top_bjd_nm.append(self.metadata.sgg_nm)
                    self.top_bjd_cd.append(self.metadata.sgg_cd)
                else:
                    print(self.metadata._print())
            if self.top_bjd_typ == "emd":
                if self.metadata.emd_cd is not None:
                    self.top_bjd_nm.append(self.metadata.emd_nm)
                    self.top_bjd_cd.append(self.metadata.emd_cd)
                else:
                    print(self.metadata._print())


    def _update_connected_bjd(self):
        pass


    def _print(self):
        print(f"typ: {self.typ}")
        print(f"bjd_cd: {self.bjd_cd}")
        print(f"bjd_nm: {self.bjd_nm}")
        print(f"full_bjd_nm: {self.full_bjd_nm}")
        print(f"metadata: {self.metadata}")
        print(f"top_bjd_typ: {self.top_bjd_typ}")
        print(f"top_bjd_cd: {self.top_bjd_cd}")
        print(f"top_bjd_nm: {self.top_bjd_nm}")
        print(f"top_bjd: {self.top_bjd}")
        print(f"bottom_bjd_cd: {self.bottom_bjd_cd}")
        print(f"bottom_bjd_nm: {self.bottom_bjd_nm}")
        print(f"bottom_bjd: {self.bottom_bjd}")
        print(f"is_smallest: {self.is_smallest}")
        print(f"base_dt: {self.base_dt_print}")


@dataclass
class BjdConnectorGraph():


    def __init__(
        self,
        base_dt: Optional[str] = None,
        is_inherit: bool = False
    ):

        if base_dt is not None:
            if not isinstance(base_dt, str):
                raise TypeError("type of object('base_dt') must be string")

            if not base_dt.isdigit():
                raise ValueError("object('base_dt') should be a string consisting of numbers")

            if len(base_dt) != 8:
                raise ValueError("object('base_dt') should be a string consisting of exactly 8(YYYYMMDD) digits")
        else: pass

        self.base_dt = base_dt
        self.is_inherit = is_inherit
        self.logger = Log('BjdConnectorGraph').stream_handler("INFO")
        self.bjd_connectors: Dict[str, BjdConnector] = dict()
        self._get_base_dt()

        CA = ConvAddr(base_dt=self.base_dt, is_inherit=True)
        self.bjd_current_df = CA.bjd_df
        self.sido_list = CA.sido_list
        self.sgg_list = CA.sgg_list
        self.emd_list = CA.emd_list
        self.ri_list = CA.ri_list
        self.multiple_word_sgg_list = CA.multiple_word_sgg_list

        self._creates_bjd_connectors()
        self._update_bjd_connectors()
        self._check_bjd_connectors_bottom_counts()


    def _find_latest_base_dt(
        self,
        base_dts: List[str]
    ) -> str:

        """
        입력된 날짜(YYYYMMDD)와 법정동 데이터 시점 리스트와 비교하여 입력된 날짜보다 과거 시점 중 최신 시점을 반환
        """

        for date in base_dts:
            if date < self.base_dt:
                return date

        # 입력된 날짜보다 작은 날짜가 없을 경우
        self.logger.info("입력된 날짜보다 이전 시점의 법정동 데이터가 존재하지 않습니다. 보유한 데이터중 최신 데이터를 적용합니다.")
        return base_dts[0]


    def _get_base_dt(self):

        """
        입력된 날짜(YYYYMMDD)와 법정동 데이터 시점 리스트와 비교하여 입력된 날짜보다 과거 시점 중 최신 시점을 반환 \n
        입력된 날짜(YYYYMMDD)가 없으면 데이터 시점 리스트 중 최신 시점을 반환
        """

        if self.is_inherit:
            return self.base_dt

        base_dts = _get_folder_names(library_name="vdutils", base_folder_path="data/bjd")
        base_dts = sorted(base_dts, reverse=True)
        try:
            if self.base_dt is None:
                self.base_dt = base_dts[0]
            else:
                self.base_dt = self._find_latest_base_dt(base_dts=base_dts)
        finally:
            self.logger.info(f"[BjdConnectorGraph] 적용 법정동 데이터 시점: {self.base_dt}")


    def _creates_bjd_connectors(self):
        for _ in self.bjd_current_df[["법정동코드", "법정동명"]].itertuples():
            bjd_cd = str(_.법정동코드)
            full_bjd_nm =str(_.법정동명)

            self.bjd_connectors[bjd_cd] = BjdConnector(
                bjd_cd=bjd_cd,
                full_bjd_nm=full_bjd_nm,
                sido_list=self.sido_list,
                sgg_list=self.sgg_list,
                emd_list=self.emd_list,
                ri_list=self.ri_list,
                multiple_word_sgg_list=self.multiple_word_sgg_list,
                base_dt=self.base_dt
            )


    def _update_bjd_connectors(self):
        for bjd_cd, bjd_connector in self.bjd_connectors.items():
            if len(bjd_connector.top_bjd_cd):
                bjd_nm = bjd_connector.bjd_nm
                for top_bjd_cd in bjd_connector.top_bjd_cd:
                    if bjd_cd not in self.bjd_connectors[top_bjd_cd].bottom_bjd_cd:
                        self.bjd_connectors[top_bjd_cd].bottom_bjd_cd.append(bjd_cd)
                    if bjd_nm not in self.bjd_connectors[top_bjd_cd].bottom_bjd_nm:
                        self.bjd_connectors[top_bjd_cd].bottom_bjd_nm.append(bjd_nm)

        for bjd_cd, bjd_connector in self.bjd_connectors.items():
            if len(bjd_connector.top_bjd_cd) > 0:
                for top_bjd_cd in bjd_connector.top_bjd_cd:
                    bjd_connector.top_bjd.append(self.bjd_connectors[top_bjd_cd])
            if len(bjd_connector.bottom_bjd_cd) > 0:
                for bottom_bjd_cd in bjd_connector.bottom_bjd_cd:
                    bjd_connector.bottom_bjd.append(self.bjd_connectors[bottom_bjd_cd])

        for bjd_cd, bjd_connector in self.bjd_connectors.items():
            if len(bjd_connector.bottom_bjd_cd) == 0: bjd_connector.is_smallest = True
            else: bjd_connector.is_smallest = False


    def _check_bjd_connectors_bottom_counts(self):
        for bjd_cd, bjd_connector in self.bjd_connectors.items():
            if len(bjd_connector.bottom_bjd_cd) != len(bjd_connector.bottom_bjd):
                # bottom_bjd_nm 은 동일한 값이 있는경우가 있어서 제외
                # 4311132026 충청북도 청주시 상당구 미원면 기암리
                # 4311132033 충청북도 청주시 상당구 미원면 기암리
                raise ValueError(f"{bjd_cd}: Count of bottom values is not same")


@dataclass
class FullBjdConnector():

    def __init__(
        self,
        full_bjd_cd: str,
        full_bjd_nm: str,
        created_dt: str,
        deleted_dt: str,
        before_bjd_cd: str,
        bjd_connectors,
        multiple_word_sgg_list,
        base_dt: str
    ):
        self.base_dt = base_dt
        self.base_dt_print: str = f"{self.base_dt[:4]}-{self.base_dt[4:6]}-{self.base_dt[6:8]}"
        self.bjd_connectors = bjd_connectors
        self.multiple_word_sgg_list = multiple_word_sgg_list
        self.full_bjd_cd: str = full_bjd_cd
        self.full_bjd_nm: str = full_bjd_nm
        self.is_exist: bool = None
        self.created_dt: Optional[str] = created_dt
        self.deleted_dt: Optional[str] = deleted_dt
        self.before_bjd_cd: Optional[str] = before_bjd_cd
        self.before: List[FullBjdConnector] = []
        self.after: List[FullBjdConnector] = []
        self.is_smallest: bool = None
        self.sido: Optional[bool] = None
        self.sgg: Optional[bool] = None
        self.emd: Optional[bool] = None
        self.ri: Optional[bool] = None
        self.sido_nm: Optional[str] = None
        self.sgg_nm: Optional[str] = None
        self.emd_nm: Optional[str] = None
        self.ri_nm: Optional[str] = None
        self.sido_cd: Optional[str] = None
        self.sgg_cd: Optional[str] = None
        self.emd_cd: Optional[str] = None
        self.ri_cd: Optional[str] = None
        self.sido_bjd_connector: Optional[BjdConnector] = None
        self.sgg_bjd_connector: Optional[BjdConnector] = None
        self.emd_bjd_connector: Optional[BjdConnector] = None
        self.ri_bjd_connector: Optional[BjdConnector] = None
        self.is_exist = self._get_is_exist()

        # CA = ConvAddr(base_dt=self.base_dt)
        # BCG = BjdConnectorGraph(base_dt=self.base_dt, is_inherit=True)
        # self.bjd_connectors = BCG.bjd_connectors
        # self.multiple_word_sgg_list = BCG.multiple_word_sgg_list

        self._get_bjd_connectors()


    def _split_full_bjd_nm(
        self
    ):
        if self.full_bjd_cd is not None:
            for multiple_sgg_nm in self.multiple_word_sgg_list:
                if multiple_sgg_nm in self.full_bjd_nm:
                    return [multiple_sgg_nm] + self.full_bjd_nm.replace(multiple_sgg_nm, "").split()
            return self.full_bjd_nm.split()
        return []


    def _get_is_exist(self):
        if self.deleted_dt is not None: return False
        return True


    def _get_bjd_cd_and_nm_from_typ(
        self, 
        bjd_connector
    ):
        typ = bjd_connector.typ
        setattr(self, f"{typ}", True)
        setattr(self, f"{typ}_cd", bjd_connector.bjd_cd)
        setattr(self, f"{typ}_nm", bjd_connector.bjd_nm)
        setattr(self, f"{typ}_bjd_connector", bjd_connector)


    def _get_each_bjd_connector(
        self,
        bjd_connector_list: List[BjdConnector], 
        full_bjd_nm_list: List[str]
    ):
        if len(bjd_connector_list) and len(full_bjd_nm_list):
            included_bjds = []
            for bjd_connector in bjd_connector_list:
                if bjd_connector.bjd_nm in full_bjd_nm_list:
                    included_bjds.append(bjd_connector.bjd_nm)
                    full_bjd_nm_list.remove(bjd_connector.bjd_nm)

                    self._get_bjd_cd_and_nm_from_typ(bjd_connector)
                    self._get_each_bjd_connector(
                        bjd_connector.top_bjd,
                        full_bjd_nm_list
                    )
            if len(included_bjds) == 0:
                raise ValueError(f"{[bjd_connector.bjd for bjd_connector in bjd_connector_list]}, Not in This Full Bjd Name List {full_bjd_nm_list}")


    def _get_bjd_connectors(self):
        full_bjd_nm_list = self._split_full_bjd_nm()
        start_bjd_connector = self.bjd_connectors[self.full_bjd_cd] # 가장 작은 단위의 법정동
        try:
            self._get_each_bjd_connector([start_bjd_connector], full_bjd_nm_list)
        except:
            print(f"Error Full Bjd Name List: {full_bjd_nm_list}, Bjd Code: {self.full_bjd_cd}")


    def _print(self):
        print(f"full_bjd_cd: {self.full_bjd_cd}")
        print(f"full_bjd_nm: {self.full_bjd_nm}")
        print(f"is_exist: {self.is_exist}")
        print(f"created_dt: {self.created_dt}")
        print(f"deleted_dt: {self.deleted_dt}")
        print(f"before: {self.before}")
        print(f"after: {self.after}")
        print(f"is_smallest: {self.is_smallest}")
        print(f"sido: {self.sido}")
        print(f"sgg: {self.sgg}")
        print(f"emd: {self.emd}")
        print(f"ri: {self.ri}")
        print(f"sido_nm: {self.sido_nm}")
        print(f"sgg_nm: {self.sgg_nm}")
        print(f"emd_nm: {self.emd_nm}")
        print(f"ri_nm: {self.ri_nm}")
        print(f"sido_cd: {self.sido_cd}")
        print(f"sgg_cd: {self.sgg_cd}")
        print(f"emd_cd: {self.emd_cd}")
        print(f"ri_cd: {self.ri_cd}")
        print(f"sido_bjd_connector: {self.sido_bjd_connector}")
        print(f"sgg_bjd_connector: {self.sgg_bjd_connector}")
        print(f"emd_bjd_connector: {self.emd_bjd_connector}")
        print(f"ri_bjd_connector: {self.ri_bjd_connector}")
        print(f"base_dt: {self.base_dt_print}")


@dataclass
class FullBjdConnectorGraph():


    def __init__(
        self,
        base_dt: Optional[str] = None,
        is_inherit: bool = False
    ):

        if base_dt is not None:
            if not isinstance(base_dt, str):
                raise TypeError("type of object('base_dt') must be string")

            if not base_dt.isdigit():
                raise ValueError("object('base_dt') should be a string consisting of numbers")

            if len(base_dt) != 8:
                raise ValueError("object('base_dt') should be a string consisting of exactly 8(YYYYMMDD) digits")
        else: pass

        self.base_dt = base_dt
        self.is_inherit = is_inherit
        self.logger = Log('FullBjdConnectorGraph').stream_handler("INFO")
        self.full_bjd_connectors: Dict[str, FullBjdConnector] = dict()
        self._get_base_dt()

        CA = ConvAddr(base_dt=self.base_dt, is_inherit=True)
        BCG = BjdConnectorGraph(base_dt=self.base_dt, is_inherit=True)
        self.bjd_df = CA.bjd_df
        self.bjd_connectors = BCG.bjd_connectors
        self.multiple_word_sgg_list = CA.multiple_word_sgg_list

        self._creates_full_bjd_connectors()
        self._update_before_and_after()


    def _find_latest_base_dt(
        self,
        base_dts: List[str]
    ) -> str:

        """
        입력된 날짜(YYYYMMDD)와 법정동 데이터 시점 리스트와 비교하여 입력된 날짜보다 과거 시점 중 최신 시점을 반환
        """

        for date in base_dts:
            if date < self.base_dt:
                return date

        # 입력된 날짜보다 작은 날짜가 없을 경우
        self.logger.info("입력된 날짜보다 이전 시점의 법정동 데이터가 존재하지 않습니다. 보유한 데이터중 최신 데이터를 적용합니다.")
        return base_dts[0]


    def _get_base_dt(self):

        """
        입력된 날짜(YYYYMMDD)와 법정동 데이터 시점 리스트와 비교하여 입력된 날짜보다 과거 시점 중 최신 시점을 반환 \n
        입력된 날짜(YYYYMMDD)가 없으면 데이터 시점 리스트 중 최신 시점을 반환
        """
        
        if self.is_inherit:
            return self.base_dt

        base_dts = _get_folder_names(library_name="vdutils", base_folder_path="data/bjd")
        base_dts = sorted(base_dts, reverse=True)
        try:
            if self.base_dt is None:
                self.base_dt = base_dts[0]
            else:
                self.base_dt = self._find_latest_base_dt(base_dts=base_dts)
        finally:
            self.logger.info(f"[FullBjdConnectorGraph] 적용 법정동 데이터 시점: {self.base_dt}")


    @staticmethod
    def _replace_nan_with_none(df: pd.DataFrame):
        return df.where(pd.notna(df), None)


    def _creates_full_bjd_connectors(self):
        self.bjd_df = self.bjd_df[[
                "과거법정동코드",
                "법정동코드",
                "삭제일자",
                "생성일자",
                "법정동명"
            ]]
        self.bjd_df = self._replace_nan_with_none(self.bjd_df)
        for _ in self.bjd_df.itertuples():
            before_bjd_cd = str(_.과거법정동코드) if _.과거법정동코드 is not None else None
            full_bjd_cd = str(_.법정동코드) if _.법정동코드 is not None else None
            full_bjd_nm = str(_.법정동명) if _.법정동명 is not None else None
            created_dt = str(_.생성일자) if _.생성일자 is not None else None
            deleted_dt = str(_.삭제일자) if _.삭제일자 is not None else None

            self.full_bjd_connectors[full_bjd_cd] = FullBjdConnector(
                full_bjd_cd=full_bjd_cd,
                full_bjd_nm=full_bjd_nm,
                created_dt=created_dt,
                deleted_dt=deleted_dt,
                before_bjd_cd=before_bjd_cd,
                bjd_connectors=self.bjd_connectors,
                multiple_word_sgg_list=self.multiple_word_sgg_list,
                base_dt=self.base_dt
            )


    def _update_before_and_after(self):
        for bjd_cd, full_bjd_connector in self.full_bjd_connectors.items():
            if full_bjd_connector.before_bjd_cd is not None \
            and full_bjd_connector.before_bjd_cd in self.full_bjd_connectors.keys():
                self.full_bjd_connectors[bjd_cd].before.append(self.full_bjd_connectors[full_bjd_connector.before_bjd_cd])
                self.full_bjd_connectors[full_bjd_connector.before_bjd_cd].after.append(self.full_bjd_connectors[bjd_cd])


@dataclass
class ConvAddrByBjdConnector():

    bjd_connectors = None
    full_bjd_connectors = None

    def __init__(
        self,
        base_dt: Optional[str] = None,
        is_inherit: bool = False
        ):

            if base_dt is not None:
                if not isinstance(base_dt, str):
                    raise TypeError("type of object('base_dt') must be string")

                if not base_dt.isdigit():
                    raise ValueError("object('base_dt') should be a string consisting of numbers")

                if len(base_dt) != 8:
                    raise ValueError("object('base_dt') should be a string consisting of exactly 8(YYYYMMDD) digits")
            else: pass

            self.base_dt = base_dt
            self.is_inherit = is_inherit
            self.logger = Log('ConvAddrByBjdConnector').stream_handler("INFO")
            self._get_base_dt()
            self.load_data()
            self.base_dt_print: str = f"{self.base_dt[:4]}-{self.base_dt[4:6]}-{self.base_dt[6:8]}"


    def _find_latest_base_dt(
        self,
        base_dts: List[str]
    ) -> str:

        """
        입력된 날짜(YYYYMMDD)와 법정동 데이터 시점 리스트와 비교하여 입력된 날짜보다 과거 시점 중 최신 시점을 반환
        """

        for date in base_dts:
            if date < self.base_dt:
                return date

        # 입력된 날짜보다 작은 날짜가 없을 경우
        self.logger.info("입력된 날짜보다 이전 시점의 법정동 데이터가 존재하지 않습니다. 보유한 데이터중 최신 데이터를 적용합니다.")
        return base_dts[0]


    def _get_base_dt(self):

        """
        입력된 날짜(YYYYMMDD)와 법정동 데이터 시점 리스트와 비교하여 입력된 날짜보다 과거 시점 중 최신 시점을 반환 \n
        입력된 날짜(YYYYMMDD)가 없으면 데이터 시점 리스트 중 최신 시점을 반환
        """

        if self.is_inherit:
            return self.base_dt

        base_dts = _get_folder_names(library_name="vdutils", base_folder_path="data/bjd")
        base_dts = sorted(base_dts, reverse=True)
        try:
            if self.base_dt is None:
                self.base_dt = base_dts[0]
            else:
                self.base_dt = self._find_latest_base_dt(base_dts=base_dts)
        finally:
            self.logger.info(f"[ConvAddrByBjdConnector] 적용 법정동 데이터 시점: {self.base_dt}")


    def load_data(self):
        FBCG = FullBjdConnectorGraph(base_dt=self.base_dt, is_inherit=True)
        if self.bjd_connectors is None:
            self.bjd_connectors = FBCG.bjd_connectors
            # with open(cls.file_name_bjd_connectors, "rb") as f:
            #     cls.bjd_connectors = pickle.load(f)
            #     print("Done loaded bjd_connectors.pkl")
        if self.full_bjd_connectors is None:
            self.full_bjd_connectors = FBCG.full_bjd_connectors
            # with open(cls.file_name_full_bjd_connectors, "rb") as f:
            #     cls.full_bjd_connectors = pickle.load(f)
            #     print("Done loaded full_bjd_connectors.pkl")


    def _get_bjd_connectors(
        self,
        addr: str,
        bjd_connector: BjdConnector
    ):
        if bjd_connector.is_smallest:
            return bjd_connector

        for bottom_bjd_connector in bjd_connector.bottom_bjd:
            if bottom_bjd_connector.bjd_nm in addr:
                result = self._get_bjd_connectors(addr, bottom_bjd_connector)
                if result is not None:
                    return result
        return None


    def _get_correct_bjd_connector(
        self,
        addr: str
    ):
        for bjd_connector in self.bjd_connectors.values():
            if bjd_connector.typ == 'sido' \
            and bjd_connector.bjd_nm in addr:
                return self._get_bjd_connectors(addr, bjd_connector)


    def _get_full_bjd_connector_by_bjd_connector(
        self,
        bjd_connector: BjdConnector
    ):
        return self.full_bjd_connectors[bjd_connector.bjd_cd]


    def _get_recently_full_bjd_connector(
        self,
        full_bjd_connector: FullBjdConnector
    ):
        if len(full_bjd_connector.after):
            if len(full_bjd_connector.after) > 1:
                print(f"{full_bjd_connector.after}가 복수개 존재합니다. 1번째 값을 이용합니다.")
            # print(f"{full_bjd_connector.full_bjd_nm} -> {full_bjd_connector.after[0].full_bjd_nm}")
            return self._get_recently_full_bjd_connector(full_bjd_connector.after[0])
        return full_bjd_connector


    def get_full_bjd_connector_by_address(
        self,
        addr: Optional[str]
    ):

        if not isinstance(addr, str):
            raise TypeError("type of object('addr') must be string")

        if addr is None:
            raise ValueError("addr is None")
            
        bjd_connector = self._get_correct_bjd_connector(addr)
        if bjd_connector is not None:
            full_bjd_connector = self._get_full_bjd_connector_by_bjd_connector(bjd_connector)
            full_bjd_connector = self._get_recently_full_bjd_connector(full_bjd_connector)
            return full_bjd_connector
        else:
            return None



import os
import sys
import unittest
from datetime import datetime
from symspellpy import SymSpell
from symspellpy.suggest_item import SuggestItem
sys.path.append(os.getcwd())
from vdutils.cordate import (
    CorDate,
    get_correct_array,
    get_correct_one
)


class TestClass(unittest.TestCase):

    @classmethod
    def setUp(cls):
        "Hook method for setting fixture before running tests in the class"
        cls.driver = 'test'
        cls.instance = CorDate()
        cls.date_1 = '19880416'
        cls.date_2 = '1988416'
        cls.date_3 = '9880416'
        cls.date_4 = '198804'
        cls.date_5 = '880416'
        cls.date_6 = '88416'
        cls.date_7 = '19884'
        cls.date_8 = '1988'
        cls.err_str = 'abc'


    @classmethod
    def tearDown(cls):
        "Hook method for deconstructing the class fixture after running all tests in the class"


    def test_sym_spell(self):
        """클래스 인스턴스 초기 생성자 sym_spell 타입 테스트 메소드"""
        self.assertIsInstance(self.instance.sym_spell, SymSpell)


    def test_dictionary_path(self):
        """클래스 인스턴스 초기 생성자 dictionary_path 타입, 값 테스트 메소드"""
        self.assertIsInstance(self.instance.dictionary_path, str)
        self.assertEqual(
            self.instance.dictionary_path,
            f'{os.getcwd()}/vdutils/data/date/date_dictionary.txt'
        )


    def test_this_year(self):
        """클래스 인스턴스 초기 생성자 this_year 타입, 값 테스트 메소드"""
        self.assertIsInstance(self.instance.this_year, int)
        self.assertEqual(self.instance.this_year, datetime.now().year)


    def test_this_year_two_length(self):
        """클래스 인스턴스 초기 생성자 this_year_tow_length 타입, 값 테스트 메소드"""
        self.assertIsInstance(self.instance.this_year_two_length, int)
        self.assertEqual(
            self.instance.this_year_two_length, 
            int(str(datetime.now().year)[2:])
        )


    def test_max_edit_distance(self):
        """클래스 인스턴스 초기 생성자 max_edit_distance 타입, 값 테스트 메소드"""
        self.assertIsInstance(self.instance.max_edit_distance, int)
        self.assertEqual(self.instance.max_edit_distance, 2)


    def test_runs(self):
        """단순 실행여부 판별하는 테스트 메소드"""

        get_correct_array(self.date_1)
        get_correct_one(self.date_1)
        self.instance.load_date_dictionary()
        self.instance.look_up_array(self.date_1)
        self.instance.look_up_one(self.date_1)
        self.instance.get_correct_array(self.date_1)
        self.instance.get_correct_one(self.date_1)


    def test_get_correct_array(self):
        """get_correct_array 함수 테스트 메소드"""

        with self.assertRaises(TypeError):
            get_correct_array(int(self.date_1))

        with self.assertRaises(ValueError):
            get_correct_array(self.err_str)

        res = get_correct_array(self.date_1)
        self.assertIsInstance(res, list)
        self.assertEqual(res, ['19880416'])


    def test_get_correct_one(self):
        """get_correct_one 함수 테스트 메소드"""

        with self.assertRaises(TypeError):
            get_correct_one(int(self.date_1))

        with self.assertRaises(ValueError):
            get_correct_array(self.err_str)

        res = get_correct_one(self.date_1)
        self.assertIsInstance(res, str)
        self.assertEqual(res, '19880416')


    def test_cls_load_date_dictionary(self):
        """load_date_dictionary 함수 테스트 메소드"""

        res = self.instance.load_date_dictionary()
        self.assertIsInstance(res, bool)
        self.assertEqual(res, True)


    def test_cls_look_up_array(self):
        """cordate.look_up_array 함수 테스트 메소드"""

        self.instance.load_date_dictionary()

        with self.assertRaises(TypeError):
            self.instance.look_up_array(int(self.date_1))

        with self.assertRaises(ValueError):
            self.instance.look_up_array(self.err_str)

        res = self.instance.look_up_array(self.date_1)
        self.assertIsInstance(res, list)
        self.assertEqual(res[0].term, '19880416')


    def test_cls_look_up_one(self):
        """cordate.look_up_one 함수 테스트 메소드"""

        self.instance.load_date_dictionary()
        
        with self.assertRaises(TypeError):
            self.instance.look_up_one(int(self.date_1))

        with self.assertRaises(ValueError):
            self.instance.look_up_one(self.err_str)

        res = self.instance.look_up_one(self.date_1)
        self.assertIsInstance(res, SuggestItem)
        self.assertEqual(res.term, '19880416')


    def test_cls_look_up_array_clean(self):
        """cordate.look_up_array_clean 함수 테스트 메소드"""

        self.instance.load_date_dictionary()

        with self.assertRaises(TypeError):
            self.instance.look_up_array_clean(int(self.date_1))

        with self.assertRaises(ValueError):
            self.instance.look_up_array_clean(self.err_str)

        res = self.instance.look_up_array_clean(self.date_1)
        self.assertIsInstance(res, list)
        self.assertEqual(res[0], '19880416')


    def test_cls_look_up_one_clean(self):
        """cordate.look_up_one_clean 함수 테스트 메소드"""

        self.instance.load_date_dictionary()
        
        with self.assertRaises(TypeError):
            self.instance.look_up_one_clean(int(self.date_1))

        with self.assertRaises(ValueError):
            self.instance.look_up_one_clean(self.err_str)

        res = self.instance.look_up_one_clean(self.date_1)
        self.assertIsInstance(res, str)
        self.assertEqual(res, '19880416')


    def test_cls_get_correct_array(self):
        """cordate.get_correct_array 함수 테스트 메소드"""

        self.instance.load_date_dictionary()
        
        with self.assertRaises(TypeError):
            self.instance.get_correct_array(int(self.date_1))

        with self.assertRaises(ValueError):
            self.instance.get_correct_array(self.err_str)
        
        res = self.instance.get_correct_array(self.date_1)
        self.assertIsInstance(res, list)
        self.assertEqual(res, ['19880416'])


    def test_cls_get_correct_one(self):
        """cordate.get_correct_one 함수 테스트 메소드"""
        
        self.instance.load_date_dictionary()
        
        with self.assertRaises(TypeError):
            self.instance.get_correct_array(int(self.date_1))

        with self.assertRaises(ValueError):
            self.instance.get_correct_array(self.err_str)

        res = self.instance.get_correct_one(self.date_1)
        self.assertIsInstance(res, str)
        self.assertEqual(res, '19880416')


    def test_correct_value(self):
        """
        get_correct_one 함수 실행결과 테스트 메소드
        cordate.get_correct_one 함수 실행결과 테스트 메소드
        """

        self.instance.load_date_dictionary()

        self.assertEqual(get_correct_one(self.date_1), '19880416')
        self.assertEqual(get_correct_one(self.date_2), '19880416')
        self.assertEqual(get_correct_one(self.date_3), '19880416')
        self.assertEqual(get_correct_one(self.date_4), '19880401')
        self.assertEqual(get_correct_one(self.date_5), '19880416')
        self.assertEqual(get_correct_one(self.date_6), '19880416')
        self.assertEqual(get_correct_one(self.date_7), '19880401')
        self.assertEqual(get_correct_one(self.date_8), '19880101')

        self.assertEqual(self.instance.get_correct_one(self.date_1), '19880416')
        self.assertEqual(self.instance.get_correct_one(self.date_2), '19880416')
        self.assertEqual(self.instance.get_correct_one(self.date_3), '19880416')
        self.assertEqual(self.instance.get_correct_one(self.date_4), '19880401')
        self.assertEqual(self.instance.get_correct_one(self.date_5), '19880416')
        self.assertEqual(self.instance.get_correct_one(self.date_6), '19880416')
        self.assertEqual(self.instance.get_correct_one(self.date_7), '19880401')
        self.assertEqual(self.instance.get_correct_one(self.date_8), '19880101')

        self.assertEqual(get_correct_one('19990101'), '19990101')
        self.assertEqual(get_correct_one('9990101'), '19990101')
        self.assertEqual(get_correct_one('990101'), '19990101')
        self.assertEqual(get_correct_one('199901'), '19990101')
        self.assertEqual(get_correct_one('199911'), '19991101')
        self.assertEqual(get_correct_one('19991'), '19990101')
        self.assertEqual(get_correct_one('9901'), '19990101')
        self.assertEqual(get_correct_one('1999'), '19990101')
        
        self.assertEqual(self.instance.get_correct_one('19990101'), '19990101')
        self.assertEqual(self.instance.get_correct_one('9990101'), '19990101')
        self.assertEqual(self.instance.get_correct_one('990101'), '19990101')
        self.assertEqual(self.instance.get_correct_one('199901'), '19990101')
        self.assertEqual(self.instance.get_correct_one('199911'), '19991101')
        self.assertEqual(self.instance.get_correct_one('19991'), '19990101')
        self.assertEqual(self.instance.get_correct_one('9901'), '19990101')
        self.assertEqual(self.instance.get_correct_one('1999'), '19990101')

        self.assertEqual(get_correct_one('20220505'), '20220505')
        self.assertEqual(get_correct_one('2022055'), '20220505')
        self.assertEqual(get_correct_one('2022505'), '20220505')
        self.assertEqual(get_correct_one('202255'), '20220505')
        self.assertEqual(get_correct_one('220505'), '20220505')
        self.assertEqual(get_correct_one('22055'), '20220505')
        self.assertEqual(get_correct_one('2255'), '20220505')
        self.assertEqual(get_correct_one('2205'), '20220501')
        self.assertEqual(get_correct_one('2022'), '20220101')
        
        self.assertEqual(self.instance.get_correct_one('20220505'), '20220505')
        self.assertEqual(self.instance.get_correct_one('2022055'), '20220505')
        self.assertEqual(self.instance.get_correct_one('2022505'), '20220505')
        self.assertEqual(self.instance.get_correct_one('202255'), '20220505')
        self.assertEqual(self.instance.get_correct_one('220505'), '20220505')
        self.assertEqual(self.instance.get_correct_one('22055'), '20220505')
        self.assertEqual(self.instance.get_correct_one('2255'), '20220505')
        self.assertEqual(self.instance.get_correct_one('2205'), '20220501')
        self.assertEqual(self.instance.get_correct_one('2022'), '20220101')
        
        self.assertEqual(get_correct_one('20230505'), '20230505')
        self.assertEqual(get_correct_one('2023055'), '20230505')
        self.assertEqual(get_correct_one('2023505'), '20230505')
        self.assertEqual(get_correct_one('202355'), '20230505')
        self.assertEqual(get_correct_one('230505'), '20230505')
        self.assertEqual(get_correct_one('23055'), '20230505')
        self.assertEqual(get_correct_one('2355'), '20230505')
        self.assertEqual(get_correct_one('2305'), '20230501')
        self.assertEqual(get_correct_one('2023'), '20230101')
        
        self.assertEqual(self.instance.get_correct_one('20230505'), '20230505')
        self.assertEqual(self.instance.get_correct_one('2023055'), '20230505')
        self.assertEqual(self.instance.get_correct_one('2023505'), '20230505')
        self.assertEqual(self.instance.get_correct_one('202355'), '20230505')
        self.assertEqual(self.instance.get_correct_one('230505'), '20230505')
        self.assertEqual(self.instance.get_correct_one('23055'), '20230505')
        self.assertEqual(self.instance.get_correct_one('2355'), '20230505')
        self.assertEqual(self.instance.get_correct_one('2305'), '20230501')
        self.assertEqual(self.instance.get_correct_one('2023'), '20230101')


if __name__ == "__main__":
    unittest.main()


import os
import sys
import unittest
import pandas as pd
from datetime import datetime
from typing import ( 
    List,
    Dict,
    Tuple,
    Optional
)
sys.path.append(os.getcwd())
from vdutils.genpnu import GenPnu


class TestClass(unittest.TestCase):

    @classmethod
    def setUp(cls):
        "Hook method for setting fixture before running tests in the class"
        cls.driver = 'test'
        cls.test_bjd_cd = '1168010500'
        cls.test_bjd_nm = '서울특별시 강남구 삼성동'
        cls.test_jibun = '1'
        cls.test_address = '서울특별시 강남구 삼성동 1'
        cls.test_base_dt = datetime.today().strftime('%Y%m%d')
        cls.instance = GenPnu(base_dt=cls.test_base_dt)


    @classmethod
    def tearDown(cls):
        "Hook method for deconstructing the class fixture after running all tests in the class"


    def test_class_initialization_type(self):
        """
        클래스 인스턴스 초기 생성자 타입 테스트 메소드

        - 클래스 인스턴스 초기 생성자 객체의 값의 타입이 지정된 타입과 일치하는지 확인하는 테스트
            - __init__.sep 객체 테스트
            - __init__.index 객체 테스트
            - __init__.encoding 객체 테스트
            - __init__.base_dt 객체 테스트
            - __init__.bjd_current_df 객체 테스트
            - __init__.bjd_current_nm_cd_dic 객체 테스트
            - __init__.bjd_dic 객체 테스트
            - __init__.bjd_nm_change_dic 객체 테스트
            - __init__.base_dt_print 객체 테스트
        """

        self.assertIsInstance(self.instance.sep, str)
        self.assertIs(self.instance.index, False) # default False
        self.assertIsInstance(self.instance.encoding, str)
        self.assertIsInstance(self.instance.base_dt, (str, type(None)))
        self.assertIsInstance(self.instance.bjd_current_df, pd.DataFrame)
        self.assertFalse(self.instance.bjd_current_df.empty) # pd.DataFrame() 형식이라 empty 확인
        self.assertIsInstance(self.instance.bjd_current_nm_cd_dic, dict)
        self.assertIsInstance(self.instance.bjd_dic, dict)
        self.assertTrue(all(isinstance(item, str) for item in self.instance.bjd_dic.keys()))
        self.assertTrue(all(isinstance(item, dict) for item in self.instance.bjd_dic.values()))
        self.assertIsInstance(self.instance.bjd_nm_change_dic, dict)
        self.assertTrue(all(isinstance(item, str) for item in self.instance.bjd_nm_change_dic.keys()))
        self.assertTrue(all(isinstance(item, str) for item in self.instance.bjd_nm_change_dic.values()))
        self.assertIsInstance(self.instance.base_dt_print, str)


    def test_class_initialization_not_empty(self):
        """
        클래스 인스턴스 초기 생성자 객체 테스트 메소드

        - 클래스 인스턴스 초기 생성자 객체의 값이 None이 아닌지(IsNotNone) 확인하는 테스트
            - __init__.sep 객체 테스트
            - __init__.index 객체 테스트
            - __init__.encoding 객체 테스트
            - __init__.base_dt 객체 테스트
            - __init__.bjd_current_df 객체 테스트
            - __init__.bjd_current_nm_cd_dic 객체 테스트
            - __init__.bjd_dic 객체 테스트
            - __init__.bjd_nm_change_dic 객체 테스트
            - __init__.base_dt_print 객체 테스트

        - 클래스 인스턴스 초기 생성자 객체의 Length가 0이 아닌지 확인하는 테스트
            - __init__.sep 객체 테스트
            - __init__.index 객체 테스트 (bool pass)
            - __init__.encoding 객체 테스트
            - __init__.base_dt 객체 테스트
            - __init__.bjd_current_df 객체 테스트
            - __init__.bjd_current_nm_cd_dic 객체 테스트
            - __init__.bjd_dic 객체 테스트
            - __init__.bjd_nm_change_dic 객체 테스트
            - __init__.base_dt_print 객체 테스트

        """

        self.assertIsNotNone(self.instance.sep)
        self.assertIsNotNone(self.instance.index)
        self.assertIsNotNone(self.instance.encoding)
        self.assertIsNotNone(self.instance.base_dt)
        self.assertIsNotNone(self.instance.bjd_current_df)
        self.assertIsNotNone(self.instance.bjd_current_nm_cd_dic)
        self.assertIsNotNone(self.instance.bjd_dic)
        self.assertIsNotNone(self.instance.bjd_nm_change_dic)
        self.assertIsNotNone(self.instance.base_dt_print)
        
        self.assertNotEqual(len(self.instance.sep), 0)
        # self.assertNotEqual(len(self.instance.index), 0) # bool 
        self.assertNotEqual(len(self.instance.encoding), 0)
        self.assertNotEqual(len(self.instance.base_dt), 0)
        self.assertNotEqual(len(self.instance.bjd_current_df), 0)
        self.assertNotEqual(len(self.instance.bjd_current_nm_cd_dic), 0)
        self.assertNotEqual(len(self.instance.bjd_dic), 0)
        self.assertNotEqual(len(self.instance.bjd_nm_change_dic), 0)
        self.assertNotEqual(len(self.instance.base_dt_print), 0)


    def test_runs(self):
        """
        단순 실행여부 판별하는 테스트 메소드

        - 클래스 인스턴스 함수 실행 확인하는 테스트
            - get_bjd_cd 함수 테스트
            - get_bjd_data 함수 테스트
            - generate_pnu 함수 테스트
            - generate_pnu_from_bjd_nm 함수 테스트
        """

        self.instance.get_bjd_cd(bjd_nm=self.test_bjd_nm)
        self.instance.get_bjd_data(bjd_cd=self.test_bjd_cd)
        self.instance.generate_pnu(bjd_cd=self.test_bjd_cd, jibun=self.test_jibun)
        self.instance.generate_pnu_from_bjd_nm(bjd_nm=self.test_bjd_nm, jibun=self.test_jibun)
        self.instance.generate_pnu_from_address(address=self.test_address)


    def test_get_bjd_cd(self):
        """
        get_bjd_cd 함수 테스트 메소드

        - get_bjd_cd 함수 예외처리 테스트
            - get_bjd_cd 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 int type)
            - get_bjd_cd 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 float type)
            - get_bjd_cd 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 None type)
            - get_bjd_cd 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 bool type)
            - get_bjd_cd 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(숫자와 한글로만 이루어진 문자열)의 문자열이 아닌 알파벳 포함)
            - get_bjd_cd 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(숫자와 한글로만 이루어진 문자열)의 문자열이 아닌 알파벳 문자열 포함)
            - get_bjd_cd 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(숫자와 한글로만 이루어진 문자열)의 문자열이 아닌 한글 자음 포함)

        - get_bjd_cd 함수 리턴값 테스트(올바른 법정동 문자열 제공)
            - get_bjd_cd 함수 리턴값 타입 테스트
            - get_bjd_cd 함수 리턴값 error 타입 테스트
            - get_bjd_cd 함수 리턴값 bjd_cd 타입 테스트
            - get_bjd_cd 함수 리턴값 deleted_dt 타입 테스트 (None pass)
            - get_bjd_cd 함수 리턴값 base_dt 타입 테스트
            - get_bjd_cd 함수 리턴값 msg 타입 테스트
            - get_bjd_cd 함수 리턴값 error 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 bjd_cd 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 deleted_dt 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 일치 테스트

        - get_bjd_cd 함수 리턴값 테스트(존재하지 않은 법정동 문자열 제공)
            - get_bjd_cd 함수 리턴값 타입 테스트
            - get_bjd_cd 함수 리턴값 error 타입 테스트
            - get_bjd_cd 함수 리턴값 bjd_cd 타입 테스트 (None pass)
            - get_bjd_cd 함수 리턴값 deleted_dt 타입 테스트 (None pass)
            - get_bjd_cd 함수 리턴값 base_dt 타입 테스트
            - get_bjd_cd 함수 리턴값 msg 타입 테스트
            - get_bjd_cd 함수 리턴값 error 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 bjd_cd 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 deleted_dt 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - get_bjd_cd 함수 리턴값 일치 테스트
        """

        with self.assertRaises(TypeError): self.instance.get_bjd_cd(bjd_nm=1)
        with self.assertRaises(TypeError): self.instance.get_bjd_cd(bjd_nm=0.1)
        with self.assertRaises(TypeError): self.instance.get_bjd_cd(bjd_nm=None)
        with self.assertRaises(TypeError): self.instance.get_bjd_cd(bjd_nm=False)
        with self.assertRaises(ValueError): self.instance.get_bjd_cd(bjd_nm='a 서울특별시 강남구 삼성동') # 알파벳 포함
        with self.assertRaises(ValueError): self.instance.get_bjd_cd(bjd_nm='seoulsi gangnamgu samsungdong') # 알파벳 포함
        with self.assertRaises(ValueError): self.instance.get_bjd_cd(bjd_nm='ㄱ 서울특별시 강남구 삼성동') # 불완전한 한글 문자열 포함

        res = self.instance.get_bjd_cd('서울특별시 강남구 삼성동') # 정상적인 법정동명 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        self.assertIsInstance(res.get('bjd_cd'), (str, type(None)))
        # self.assertTrue(res.get('deleted_dt'), Optional[str])
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNotNone(res.get('bjd_cd'))
        self.assertIsNone(res.get('deleted_dt')) # 현재 존재하는 법정동의 경우 deleted_dt == None
        self.assertIsNotNone(res.get('base_dt'))
        self.assertIsNotNone(res.get('msg'))
        self.assertEqual(res, {
            'error': False,
            'bjd_cd': '1168010500',
            'deleted_dt': None,
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': ''
        })

        res = self.instance.get_bjd_cd('부산광역시 강남구 삼성동') # 비정상적인 법정동명 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        # self.assertTrue(res.get('bjd_cd'), Optional[str])
        # self.assertTrue(res.get('deleted_dt'), Optional[str])
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNone(res.get('bjd_cd')) # 존재하지 않는 법정동의 경우, bjd_cd == None
        self.assertIsNone(res.get('deleted_dt')) # 존재하지 않는 법정동의 경우, deleted_dt == None
        self.assertIsNotNone(res.get('base_dt'))
        self.assertIsNotNone(res.get('msg'))
        self.assertEqual(res, {
            'error': True,
            'bjd_cd': None,
            'deleted_dt': None,
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': "'부산광역시 강남구 삼성동' is not a valid legal district name"
        })


    def test_get_bjd_data(self):
        """
        get_bjd_data 함수 테스트 메소드

        - get_bjd_data 함수 예외처리 테스트
            - get_bjd_data 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 int type)
            - get_bjd_data 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 float type)
            - get_bjd_data 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 None type)
            - get_bjd_data 함수 예외처리 테스트(TypeError 입력값이 문자열이 아닌 bool type)
            - get_bjd_data 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 한글 포함)
            - get_bjd_data 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 알파벳 문자열)
            - get_bjd_data 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 9자리 숫자문자열)
            - get_bjd_data 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 11자리 숫자문자열)
            - get_bjd_data 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 알파벳 포함)
            - get_bjd_data 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 공백 포함)
            - get_bjd_data 함수 예외처리 테스트(ValueError 입력값이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 특수문자 포함)

        - get_bjd_data 함수 리턴값 테스트(올바른 법정동 문자열 제공)
            - get_bjd_data 함수 리턴값 타입 테스트
            - get_bjd_data 함수 리턴값 error 타입 테스트
            - get_bjd_data 함수 리턴값 sido_nm 타입 테스트
            - get_bjd_data 함수 리턴값 sgg_nm 타입 테스트
            - get_bjd_data 함수 리턴값 emd_nm 타입 테스트
            - get_bjd_data 함수 리턴값 ri_nm 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 full_bjd_nm 타입 테스트
            - get_bjd_data 함수 리턴값 created_dt 타입 테스트
            - get_bjd_data 함수 리턴값 deleted_dt 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 base_dt 타입 테스트
            - get_bjd_data 함수 리턴값 error 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 sido_nm 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 sgg_nm 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 emd_nm 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 ri_nm 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 full_bjd_nm 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 created_dt 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 deleted_dt 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 일치 테스트

        - get_bjd_data 함수 리턴값 테스트(존재하지 않은 법정동 문자열 제공)
            - get_bjd_data 함수 리턴값 타입 테스트
            - get_bjd_data 함수 리턴값 error 타입 테스트
            - get_bjd_data 함수 리턴값 sido_nm 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 sgg_nm 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 emd_nm 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 ri_nm 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 full_bjd_nm 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 created_dt 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 deleted_dt 타입 테스트 (None pass)
            - get_bjd_data 함수 리턴값 base_dt 타입 테스트
            - get_bjd_data 함수 리턴값 error 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 sido_nm 값이 None 이 맞는지 테스트
            - get_bjd_data 함수 리턴값 sgg_nm 값이 None 이 맞는지 테스트
            - get_bjd_data 함수 리턴값 emd_nm 값이 None 이 맞는지 테스트
            - get_bjd_data 함수 리턴값 ri_nm 값이 None 이 맞는지 테스트
            - get_bjd_data 함수 리턴값 full_bjd_nm 값이 None 이 맞는지 테스트
            - get_bjd_data 함수 리턴값 created_dt 값이 None 이 맞는지 테스트
            - get_bjd_data 함수 리턴값 deleted_dt 값이 None 이 맞는지 테스트
            - get_bjd_data 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - get_bjd_data 함수 리턴값 일치 테스트
        """

        with self.assertRaises(TypeError): self.instance.get_bjd_data(bjd_cd=1)
        with self.assertRaises(TypeError): self.instance.get_bjd_data(bjd_cd=0.1)
        with self.assertRaises(TypeError): self.instance.get_bjd_data(bjd_cd=None)
        with self.assertRaises(TypeError): self.instance.get_bjd_data(bjd_cd=False)
        with self.assertRaises(ValueError): self.instance.get_bjd_data(bjd_cd='서울특별시 강남구 삼성동') # 숫자 문자열이 아닌 문자열
        with self.assertRaises(ValueError): self.instance.get_bjd_data(bjd_cd='seoulsi gangnamgu samsungdong') # 숫자 문자열이 아닌 문자열
        with self.assertRaises(ValueError): self.instance.get_bjd_data(bjd_cd='123456789') # 10자리가 아닌 숫자 문자열
        with self.assertRaises(ValueError): self.instance.get_bjd_data(bjd_cd='12345678910') # 10자리가 아닌 숫자 문자열
        with self.assertRaises(ValueError): self.instance.get_bjd_data(bjd_cd='123456789A') # 10자리이지만 알파벳이 포함된 문자열
        with self.assertRaises(ValueError): self.instance.get_bjd_data(bjd_cd='123456789 ') # 10자리이지만 공백이 포함된 문자열
        with self.assertRaises(ValueError): self.instance.get_bjd_data(bjd_cd='123456789#') # 10자리이지만 특수문자가 포함된 문자열

        res = self.instance.get_bjd_data(bjd_cd='1168010500') # 정상적인 법정동코드 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        self.assertIsInstance(res.get('sido_nm'), (str, type(None)))
        self.assertIsInstance(res.get('sgg_nm'), (str, type(None)))
        self.assertIsInstance(res.get('emd_nm'), (str, type(None)))
        # self.assertTrue(res.get('ri_nm'), Optional[str])
        self.assertIsInstance(res.get('full_bjd_nm'), (str, type(None)))
        self.assertIsInstance(res.get('created_dt'), (str, type(None)))
        # self.assertTrue(res.get('deleted_dt'), Optional[str])
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'), bool)
        self.assertIsNotNone(res.get('sido_nm'), Optional[str])
        self.assertIsNotNone(res.get('sgg_nm'), Optional[str])
        self.assertIsNotNone(res.get('emd_nm'), Optional[str])
        self.assertIsNone(res.get('ri_nm'), Optional[str]) # 테스트 법정동코드 1168010500 에는 리가 없으므로 == None
        self.assertIsNotNone(res.get('full_bjd_nm'), Optional[str])
        self.assertIsNotNone(res.get('created_dt'), Optional[str])
        self.assertIsNone(res.get('deleted_dt'), Optional[str]) # 현재 존재하는 법정동의 경우 deleted_dt == None
        self.assertIsNotNone(res.get('base_dt'), str)
        self.assertEqual(res, {
            'error': False,
            'sido_nm': '서울특별시',
            'sgg_nm': '강남구',
            'emd_nm': '삼성동',
            'ri_nm': None,
            'full_bjd_nm': '서울특별시 강남구 삼성동',
            'created_dt': '1988-04-23',
            'deleted_dt': None,
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': ''
        })

        res = self.instance.get_bjd_data(bjd_cd='1234567890') # 비정상적인 법정동코드 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        # self.assertTrue(res.get('sido_nm'), Optional[str])
        # self.assertTrue(res.get('sgg_nm'), Optional[str])
        # self.assertTrue(res.get('emd_nm'), Optional[str])
        # self.assertTrue(res.get('ri_nm'), Optional[str])
        # self.assertTrue(res.get('full_bjd_nm'), Optional[str])
        # self.assertTrue(res.get('created_dt'), Optional[str])
        # self.assertTrue(res.get('deleted_dt'), Optional[str])
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'), bool)
        self.assertIsNone(res.get('sido_nm'), Optional[str]) # 비정상적인 법정동코드의 경우 None
        self.assertIsNone(res.get('sgg_nm'), Optional[str]) # 비정상적인 법정동코드의 경우 None
        self.assertIsNone(res.get('emd_nm'), Optional[str]) # 비정상적인 법정동코드의 경우 None
        self.assertIsNone(res.get('ri_nm'), Optional[str]) # 비정상적인 법정동코드의 경우 None
        self.assertIsNone(res.get('full_bjd_nm'), Optional[str]) # 비정상적인 법정동코드의 경우 None
        self.assertIsNone(res.get('created_dt'), Optional[str]) # 비정상적인 법정동코드의 경우 None
        self.assertIsNone(res.get('deleted_dt'), Optional[str]) # 현재 존재하는 법정동의 경우 deleted_dt == None
        self.assertIsNotNone(res.get('base_dt'), str)
        self.assertEqual(res, {
            'error': True,
            'sido_nm': None,
            'sgg_nm': None,
            'emd_nm': None,
            'ri_nm': None,
            'full_bjd_nm': None,
            'created_dt': None,
            'deleted_dt': None,
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': "'1234567890' is not a valid legal district code"
        })


    def sub_test_validate_jibun(
        self,
        jibun: str,
        result: bool
    ):
        """
        _validate_jibun 함수 테스트 서브 메소드
            - assertIsInstance() 테스트
            - assertIsNotNone() 테스트
            - assertEqual() 테스트
        """
        res = self.instance._validate_jibun(jibun)
        self.assertIsInstance(res.get("error"), bool)
        self.assertIsInstance(res.get("msg"), str)
        self.assertIsNotNone(res)
        self.assertIsNotNone(res.get("error"))
        self.assertIsNotNone(res.get("msg"))
        self.assertEqual(res.get("error"), result)


    def test_validate_jibun(self):
        """
        _validate_jibun 함수 테스트 메소드

        - _validate_jibun 함수 예외처리 테스트(올바르지 않은 지번 형식 문자열 제공)
            - _validate_jibun 함수 예외처리 테스트(ValueError 입력값이 적절한 형식의 문자열이 아닌 경우)
            - _validate_jibun 함수 jibun 입력값 적절한 형태 = re.compile(r'^(산\s*)?\d{1,4}-\d{1,4}$|^(산\s*)?\d{1,4}$|^\d{1,4}-\d{1,4}$|^\d{1,4}$')

        - _validate_jibun 함수 리턴값 테스트(올바른 지번 형식 문자열 제공)
        """

        self.sub_test_validate_jibun(jibun='산 00000', result=True)
        self.sub_test_validate_jibun(jibun='산 000000', result=True)
        self.sub_test_validate_jibun(jibun='산 0000000', result=True)
        self.sub_test_validate_jibun(jibun='산 00000000', result=True)
        self.sub_test_validate_jibun(jibun='산 000000000', result=True)
        self.sub_test_validate_jibun(jibun='산 0000000000', result=True)
        self.sub_test_validate_jibun(jibun='산 11111-2', result=True)
        self.sub_test_validate_jibun(jibun='산 33333-44', result=True)
        self.sub_test_validate_jibun(jibun='산 55555-666', result=True)
        self.sub_test_validate_jibun(jibun='산 77777-8888', result=True)
        self.sub_test_validate_jibun(jibun='산 99999-00000', result=True)
        self.sub_test_validate_jibun(jibun='산 7777-88888', result=True)
        self.sub_test_validate_jibun(jibun='산 555-66666', result=True)
        self.sub_test_validate_jibun(jibun='산 33-44444', result=True)
        self.sub_test_validate_jibun(jibun='산 1-22222', result=True)
        self.sub_test_validate_jibun(jibun='산00000', result=True)
        self.sub_test_validate_jibun(jibun='산000000', result=True)
        self.sub_test_validate_jibun(jibun='산0000000', result=True)
        self.sub_test_validate_jibun(jibun='산00000000', result=True)
        self.sub_test_validate_jibun(jibun='산000000000', result=True)
        self.sub_test_validate_jibun(jibun='산0000000000', result=True)
        self.sub_test_validate_jibun(jibun='산11111-2', result=True)
        self.sub_test_validate_jibun(jibun='산33333-44', result=True)
        self.sub_test_validate_jibun(jibun='산55555-666', result=True)
        self.sub_test_validate_jibun(jibun='산77777-8888', result=True)
        self.sub_test_validate_jibun(jibun='산99999-00000', result=True)
        self.sub_test_validate_jibun(jibun='산7777-88888', result=True)
        self.sub_test_validate_jibun(jibun='산555-66666', result=True)
        self.sub_test_validate_jibun(jibun='산33-44444', result=True)
        self.sub_test_validate_jibun(jibun='산1-22222', result=True)
        self.sub_test_validate_jibun(jibun='산-1000', result=True)
        self.sub_test_validate_jibun(jibun='산 -1000', result=True)
        self.sub_test_validate_jibun(jibun='산  -1000', result=True)
        self.sub_test_validate_jibun(jibun='산   -1000', result=True)
        self.sub_test_validate_jibun(jibun='산1000-', result=True)
        self.sub_test_validate_jibun(jibun='산 1000-', result=True)
        self.sub_test_validate_jibun(jibun='산  1000-', result=True)
        self.sub_test_validate_jibun(jibun='산   1000-', result=True)
        self.sub_test_validate_jibun(jibun='산 10000', result=True)
        self.sub_test_validate_jibun(jibun='a', result=True)
        self.sub_test_validate_jibun(jibun='a000', result=True)
        self.sub_test_validate_jibun(jibun='000a', result=True)
        self.sub_test_validate_jibun(jibun='a-1000', result=True)
        self.sub_test_validate_jibun(jibun='1000-a', result=True)
        self.sub_test_validate_jibun(jibun='ㄱ', result=True)
        self.sub_test_validate_jibun(jibun='ㄱ000', result=True)
        self.sub_test_validate_jibun(jibun='000ㄱ', result=True)
        self.sub_test_validate_jibun(jibun='ㄱ-1000', result=True)
        self.sub_test_validate_jibun(jibun='1000-ㄱ', result=True)
        self.sub_test_validate_jibun(jibun='나', result=True)
        self.sub_test_validate_jibun(jibun='나000', result=True)
        self.sub_test_validate_jibun(jibun='000나', result=True)
        self.sub_test_validate_jibun(jibun='나-1000', result=True)
        self.sub_test_validate_jibun(jibun='1000-나', result=True)
        self.sub_test_validate_jibun(jibun='선 1000-1000', result=True)
        self.sub_test_validate_jibun(jibun='순 1000-1000', result=True)
        self.sub_test_validate_jibun(jibun='상 1000-1000', result=True)
        self.sub_test_validate_jibun(jibun='선산 1000-1000', result=True)
        self.sub_test_validate_jibun(jibun='나산 1000-1000', result=True)
        self.sub_test_validate_jibun(jibun='산나 1000-1000', result=True)
        self.sub_test_validate_jibun(jibun='산산 1000-1000', result=True)
        self.sub_test_validate_jibun(jibun='산산산 1000-1000', result=True)


    def test_get_mountain_cd(self):
        """
        _get_mountain_cd 함수 테스트 메소드

        - _get_mountain_cd 함수 리턴값 테스트(올바른 지번 형식 문자열 제공)
        """

        res = self.instance._get_mountain_cd(jibun='산0000-0000')
        self.assertIsInstance(res, tuple)
        self.assertIsNotNone(res)
        self.assertEqual(res, ('0000-0000', '2'))

        res = self.instance._get_mountain_cd(jibun='0000-0000')
        self.assertIsInstance(res, tuple)
        self.assertIsNotNone(res)
        self.assertEqual(res, ('0000-0000', '1'))


    def sub_test_get_jibun_datas(
        self,
        jibun: str,
        result: Tuple[str]
    ):
        """
        _get_jibun_datas 함수 테스트 서브 메소드
            - assertIsInstance() 테스트(리턴값, tuple 개별 객체)
            - assertIsNotNone() 테스트(리턴값, tuple 개별 객체)
            - assertEqual() 테스트
        """
        res = self.instance._get_jibun_datas(jibun)
        self.assertIsInstance(res, tuple)
        self.assertIsNotNone(res)
        for item in res:
            self.assertIsNotNone(item)
            self.assertIsInstance(item, str) 
        self.assertEqual(res, result)


    def test_get_jibun_datas(self):
        """
        _get_jibun_datas 함수 테스트 메소드

        - _get_jibun_datas 함수 리턴값 테스트(올바른 지번 형식 문자열 제공)
        """

        self.sub_test_get_jibun_datas(jibun='0', result=('00000000', '0', '0'))
        self.sub_test_get_jibun_datas(jibun='0', result=('00000000', '0', '0'))
        self.sub_test_get_jibun_datas(jibun='0-0', result=('00000000', '0', '0'))
        self.sub_test_get_jibun_datas(jibun='0000', result=('00000000', '0', '0'))
        self.sub_test_get_jibun_datas(jibun='0000-0000', result=('00000000', '0', '0'))
        self.sub_test_get_jibun_datas(jibun='1', result=('00010000', '1', '0'))
        self.sub_test_get_jibun_datas(jibun='1-1', result=('00010001', '1', '1'))
        self.sub_test_get_jibun_datas(jibun='1111', result=('11110000', '1111', '0'))
        self.sub_test_get_jibun_datas(jibun='1111-1111', result=('11111111', '1111', '1111'))
        self.sub_test_get_jibun_datas(jibun='10', result=('00100000', '10', '0'))
        self.sub_test_get_jibun_datas(jibun='10-10', result=('00100010', '10', '10'))
        self.sub_test_get_jibun_datas(jibun='101', result=('01010000', '101', '0'))
        self.sub_test_get_jibun_datas(jibun='101-101', result=('01010101', '101', '101'))
        self.sub_test_get_jibun_datas(jibun='1010', result=('10100000', '1010', '0'))
        self.sub_test_get_jibun_datas(jibun='1010-1010', result=('10101010', '1010', '1010'))


    def sub_test_validate_pnu(
        self,
        pnu,
        result: dict
    ):
        """
        _validate_pnu 함수 테스트 서브 메소드
            - assertIsInstance() 테스트(리턴값, dict)
            - assertIsNotNone() 테스트(리턴값, dict 개별 객체)
            - assertEqual() 테스트
        """
        res = self.instance._validate_pnu(pnu)
        self.assertIsInstance(res, dict)
        self.assertIsNotNone(res)
        self.assertIsInstance(res.get("error"), bool)
        self.assertIsInstance(res.get("msg"), str)
        self.assertIsNotNone(res.get("error"))
        self.assertIsNotNone(res.get("msg"))
        self.assertEqual(res, result)


    def test_validate_pnu(self):
        """
        _validate_pnu 함수 테스트 메소드

        - _validate_pnu 함수 예외처리 테스트(올바르지 않은 지번 형식 문자열 제공)
            - _validate_pnu 함수 예외처리 테스트(ValueError 입력값이 적절한 형식의 문자열이 아닌 경우)
            - _validate_pnu 함수 pnu 입력값 적절한 형태 re.compile(r'^\d{19}$')

        - _validate_pnu 함수 리턴값 테스트(올바른 pnu 형식 문자열 제공)
        """

        self.sub_test_validate_pnu(
            pnu=1234567890123456789,
            result = {
                'error': True,
                'msg': "Invalid 'pnu' format. Please follow the specified format."
            }
        )
        self.sub_test_validate_pnu(
            pnu='12345678901234567890',
            result = {
                'error': True,
                'msg': "Invalid 'pnu' format. Please follow the specified format."
            }
        )
        self.sub_test_validate_pnu(
            pnu='1234567890123456789a',
            result = {
                'error': True,
                'msg': "Invalid 'pnu' format. Please follow the specified format."
            }
        )
        self.sub_test_validate_pnu(
            pnu='1234567890123456789',
            result = {
                'error': False,
                'msg': ''
            }
        )


    def sub_test_clean_bracket_and_content(
        self,
        string: str,
        result: str
    ):
        """
        _clean_bracket_and_content 함수 테스트 서브 메소드
            - assertIsInstance() 테스트(리턴값, str)
            - assertIsNotNone() 테스트(리턴값)
            - assertEqual() 테스트
        """

        res = self.instance._clean_bracket_and_content(string)
        self.assertIsInstance(res, str)
        self.assertIsNotNone(res)
        self.assertEqual(res, result)


    def test_clean_bracket_and_content(self):
        """
        _clean_bracket_and_content 함수 테스트 메소드
        """

        self.sub_test_clean_bracket_and_content(string='서울특별시 강남구 삼성동 1 (삼성동)', result='서울특별시 강남구 삼성동 1')
        self.sub_test_clean_bracket_and_content(string='서울특별시 강남구 삼성동 1 ()', result='서울특별시 강남구 삼성동 1')
        self.sub_test_clean_bracket_and_content(string='서울특별시 강남구 삼성동 1 [삼성동]', result='서울특별시 강남구 삼성동 1')
        self.sub_test_clean_bracket_and_content(string='서울특별시 강남구 삼성동 1 []', result='서울특별시 강남구 삼성동 1')


    def sub_test_split_main_and_detail_address(
        self,
        address: str,
        result: dict
    ):
        """
        _split_main_and_detail_address 함수 테스트 서브 메소드
            - assertIsInstance() 테스트(리턴값, dict)
            - assertIsNotNone() 테스트(리턴값, dict 개별 객체)
            - assertEqual() 테스트
        """
        res = self.instance._split_main_and_detail_address(address)
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get("error"), bool)
        self.assertIsInstance(res.get("msg"), str)
        self.assertIsNotNone(res)
        self.assertIsNotNone(res.get("error"))
        self.assertIsNotNone(res.get("msg"))
        self.assertEqual(res, result)


    def test_split_main_and_detail_address(self):
        """
        _split_main_and_detail_address 함수 테스트 메소드
        """

        self.sub_test_split_main_and_detail_address(
            address='서울특별시 강남구 삼성동 1 에이빌딩 비동 씨호',
            result={
                'error': False,
                'main_address': '서울특별시 강남구 삼성동 1',
                'etc_main_address': '',
                'detail_address': '에이빌딩 비동 씨호',
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''
            }
        )
        self.sub_test_split_main_and_detail_address(
            address='서울특별시 강남구 삼성동 1-1 에이빌딩 비동 씨호',
            result={
                'error': False,
                'main_address': '서울특별시 강남구 삼성동 1-1',
                'etc_main_address': '',
                'detail_address': '에이빌딩 비동 씨호',
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''
            }
        )
        self.sub_test_split_main_and_detail_address(
            address='서울특별시 강남구 삼성동 1 외 1필지 에이빌딩 비동 씨호',
            result={
                'error': False,
                'main_address': '서울특별시 강남구 삼성동 1',
                'etc_main_address': '외 1필지 ',
                'detail_address': '에이빌딩 비동 씨호',
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''
            }
        )


    def sub_test_extract_bjd_from_address(
        self,
        main_address: str,
        result: str
    ):
        """
        _extract_bjd_from_address 함수 테스트 서브 메소드
            - assertIsInstance() 테스트(리턴값, str)
            - assertIsNotNone() 테스트(리턴값)
            - assertEqual() 테스트
        """
        res = self.instance._extract_bjd_from_address(main_address)
        self.assertIsInstance(res, str)
        self.assertIsNotNone(res)
        self.assertEqual(res, result)


    def test_extract_bjd_from_address(self):
        """
        _extract_bjd_from_address 함수 테스트 메소드
        """

        self.sub_test_extract_bjd_from_address(main_address='서울특별시 강남구 삼성동 1', result='서울특별시 강남구 삼성동')
        self.sub_test_extract_bjd_from_address(main_address='서울특별시 강남구 삼성동 1-1', result='서울특별시 강남구 삼성동')
        self.sub_test_extract_bjd_from_address(main_address='서울특별시 강남구 삼성동 산 1', result='서울특별시 강남구 삼성동')
        self.sub_test_extract_bjd_from_address(main_address='서울특별시 강남구 삼성동 산 1-1', result='서울특별시 강남구 삼성동')
        self.sub_test_extract_bjd_from_address(main_address='서울특별시 강남구 삼성동 산1', result='서울특별시 강남구 삼성동')
        self.sub_test_extract_bjd_from_address(main_address='서울특별시 강남구 삼성동 산1-1', result='서울특별시 강남구 삼성동')


    def sub_test_generate_pnu(
        self,
        bjd_cd,
        jibun,
        result: bool
    ):
        """
        _generate_pnu 함수 테스트 서브 메소드
            - assertIsInstance() 테스트
            - assertIsNotNone() 테스트
            - assertEqual() 테스트
        """
        res = self.instance.generate_pnu(bjd_cd, jibun)
        self.assertIsInstance(res.get("error"), bool)
        self.assertIsInstance(res.get("msg"), str)
        self.assertIsNotNone(res)
        self.assertIsNotNone(res.get("error"))
        self.assertIsNotNone(res.get("msg"))
        self.assertEqual(res.get("error"), result)


    def test_generate_pnu(self):
        """
        generate_pnu 함수 테스트 메소드

        - generate_pnu 함수 예외처리 테스트
            - generate_pnu 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 int type)
            - generate_pnu 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 float type)
            - generate_pnu 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 None type)
            - generate_pnu 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 bool type)
            - generate_pnu 함수 예외처리 테스트(ValueError 입력값(bjd_cd)이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 경우들)
            - generate_pnu 함수 예외처리 테스트(ValueError 입력값(jibun)이 적절한 형식의 문자열이 아닌 경우들)

        - generate_pnu 함수 리턴값 테스트(올바른 법정동 문자열 및 지번 제공)
            - generate_pnu 함수 리턴값 타입 테스트
            - generate_pnu 함수 리턴값 error 타입 테스트
            - generate_pnu 함수 리턴값 pnu 타입 테스트
            - generate_pnu 함수 리턴값 bjd_cd 타입 테스트
            - generate_pnu 함수 리턴값 mountain_cd 타입 테스트
            - generate_pnu 함수 리턴값 bunji_cd 타입 테스트
            - generate_pnu 함수 리턴값 bjd_datas 타입 테스트
            - generate_pnu 함수 리턴값 bun 타입 테스트
            - generate_pnu 함수 리턴값 ji 타입 테스트
            - generate_pnu 함수 리턴값 msg 타입 테스트
            - generate_pnu 함수 리턴값 base_dt 타입 테스트
            - generate_pnu 함수 리턴값 error 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 pnu 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bjd_cd 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 mountain_cd 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bunji_cd 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bjd_datas 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bun 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 ji 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 일치 테스트

        - generate_pnu 함수 리턴값 테스트(형식은 맞지만 존재하지 않는 법정동 문자열 및 정상적인 지번 제공)
            - generate_pnu 함수 리턴값 타입 테스트
            - generate_pnu 함수 리턴값 error 타입 테스트
            - generate_pnu 함수 리턴값 pnu 타입 테스트
            - generate_pnu 함수 리턴값 bjd_cd 타입 테스트
            - generate_pnu 함수 리턴값 mountain_cd 타입 테스트
            - generate_pnu 함수 리턴값 bunji_cd 타입 테스트
            - generate_pnu 함수 리턴값 bjd_datas 타입 테스트
            - generate_pnu 함수 리턴값 bun 타입 테스트
            - generate_pnu 함수 리턴값 ji 타입 테스트
            - generate_pnu 함수 리턴값 msg 타입 테스트
            - generate_pnu 함수 리턴값 base_dt 타입 테스트
            - generate_pnu 함수 리턴값 error 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 pnu 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bjd_cd 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 mountain_cd 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bunji_cd 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bjd_datas 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 bun 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 ji 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - generate_pnu 함수 리턴값 일치 테스트
        """

        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd=0, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd=0.1, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd=None, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd=True, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd=False, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd='', jibun=0)
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd='', jibun=0.1)
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd='', jibun=None)
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd='', jibun=True)
        with self.assertRaises(TypeError): self.instance.generate_pnu(bjd_cd='', jibun=False)

        with self.assertRaises(ValueError): self.instance.generate_pnu(bjd_cd='서울특별시 강남구 삼성동', jibun='0000-0000') # 숫자 문자열이 아닌 문자열
        with self.assertRaises(ValueError): self.instance.generate_pnu(bjd_cd='seoulsi gangnamgu samsungdong', jibun='0000-0000') # 숫자 문자열이 아닌 문자열
        with self.assertRaises(ValueError): self.instance.generate_pnu(bjd_cd='123456789', jibun='0000-0000') # 10자리가 아닌 숫자 문자열
        with self.assertRaises(ValueError): self.instance.generate_pnu(bjd_cd='12345678910', jibun='0000-0000') # 10자리가 아닌 숫자 문자열
        with self.assertRaises(ValueError): self.instance.generate_pnu(bjd_cd='123456789A', jibun='0000-0000') # 10자리이지만 알파벳이 포함된 문자열
        with self.assertRaises(ValueError): self.instance.generate_pnu(bjd_cd='123456789', jibun='0000-0000') # 10자리이지만 공백이 포함된 문자열
        with self.assertRaises(ValueError): self.instance.generate_pnu(bjd_cd='123456789', jibun='0000-0000') # 10자리이지만 특수문자가 포함된 문자열

        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 00000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 0000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 00000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 000000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 0000000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 11111-2', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 33333-44', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 55555-666', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 77777-8888', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 99999-00000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 7777-88888', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 555-66666', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 33-44444', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 1-22222', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산00000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산0000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산00000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산000000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산0000000000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산11111-2', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산33333-44', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산55555-666', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산77777-8888', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산99999-00000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산7777-88888', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산555-66666', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산33-44444', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산1-22222', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 -1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산  -1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산   -1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산1000-', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 1000-', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산  1000-', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산   1000-', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산 10000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='a', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='a000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='000a', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='a-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='1000-a', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='ㄱ', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='ㄱ000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='000ㄱ', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='ㄱ-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='1000-ㄱ', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='나', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='나000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='000나', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='나-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='1000-나', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='선 1000-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='순 1000-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='상 1000-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='선산 1000-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='나산 1000-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산나 1000-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산산 1000-1000', result=True)
        self.sub_test_generate_pnu(bjd_cd='1168010500', jibun='산산산 1000-1000', result=True)

        res = self.instance.generate_pnu(bjd_cd='1168010500', jibun='1-1') # 정상적인 법정동코드 및 지번 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        self.assertIsInstance(res.get('pnu'), str)
        self.assertIsInstance(res.get('bjd_cd'), str)
        self.assertIsInstance(res.get('mountain_cd'), str)
        self.assertIsInstance(res.get('bunji_cd'), str)
        self.assertIsInstance(res.get('bjd_datas'), dict)
        self.assertIsInstance(res.get('bun'), str)
        self.assertIsInstance(res.get('ji'), str)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNotNone(res.get('pnu'))
        self.assertIsNotNone(res.get('bjd_cd'))
        self.assertIsNotNone(res.get('mountain_cd'))
        self.assertIsNotNone(res.get('bunji_cd'))
        self.assertIsNotNone(res.get('bjd_datas'))
        self.assertIsNotNone(res.get('bun'))
        self.assertIsNotNone(res.get('ji'))
        self.assertIsNotNone(res.get('base_dt'))
        self.assertIsNotNone(res.get('msg'))
        self.assertEqual(res, {
            'error': False,
            'pnu': '1168010500100010001',
            'bjd_cd': '1168010500',
            'mountain_cd': '1',
            'bunji_cd': '00010001',
            'bjd_datas': {
                'error': False,
                'sido_nm': '서울특별시',
                'sgg_nm': '강남구',
                'emd_nm': '삼성동',
                'ri_nm': None,
                'full_bjd_nm': '서울특별시 강남구 삼성동',
                'created_dt': '1988-04-23',
                'deleted_dt': None,
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''
            },
            'bun': '1',
            'ji': '1',
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': '',
        })

        res = self.instance.generate_pnu(bjd_cd='1234567890', jibun='1-1') # 형식은 맞지만 존재하지않는 법정동코드 및 정상적인 지번 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        # self.assertIsInstance(res.get('pnu'), None)
        # self.assertIsInstance(res.get('bjd_cd'), None)
        # self.assertIsInstance(res.get('mountain_cd'), None)
        # self.assertIsInstance(res.get('bunji_cd'), None)
        self.assertIsInstance(res.get('bjd_datas'), dict)
        # self.assertIsInstance(res.get('bun'), None)
        # self.assertIsInstance(res.get('ji'), None)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNone(res.get('pnu'))
        self.assertIsNone(res.get('bjd_cd'))
        self.assertIsNone(res.get('mountain_cd'))
        self.assertIsNone(res.get('bunji_cd'))
        self.assertIsNotNone(res.get('bjd_datas'))
        self.assertIsNone(res.get('bun'))
        self.assertIsNone(res.get('ji'))
        self.assertIsNotNone(res.get('base_dt'))
        self.assertIsNotNone(res.get('msg'))
        self.assertEqual(res, {
            'error': True,
            'pnu': None,
            'bjd_cd': None,
            'mountain_cd': None,
            'bunji_cd': None,
            'bjd_datas': {
                'error': True,
                'sido_nm': None,
                'sgg_nm': None,
                'emd_nm': None,
                'ri_nm': None,
                'full_bjd_nm': None,
                'created_dt': None,
                'deleted_dt': None,
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': "'1234567890' is not a valid legal district code"
            },
            'bun': None,
            'ji': None,
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': "'1234567890' is not a valid legal district code",
        })


    def sub_test_generate_pnu_from_bjd_nm(
        self,
        bjd_nm,
        jibun,
        result: bool
    ):
        """
        _generate_pnu 함수 테스트 서브 메소드
            - assertIsInstance() 테스트
            - assertIsNotNone() 테스트
            - assertEqual() 테스트
        """
        res = self.instance.generate_pnu_from_bjd_nm(bjd_nm, jibun)
        self.assertIsInstance(res.get("error"), bool)
        self.assertIsInstance(res.get("msg"), str)
        self.assertIsNotNone(res)
        self.assertIsNotNone(res.get("error"))
        self.assertIsNotNone(res.get("msg"))
        self.assertEqual(res.get("error"), result)


    def test_generate_pnu_from_bjd_nm(self):
        """
        generate_pnu_from_bjd_nm 함수 테스트 메소드

        - generate_pnu_from_bjd_nm 함수 예외처리 테스트
            - generate_pnu_from_bjd_nm 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 int type)
            - generate_pnu_from_bjd_nm 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 float type)
            - generate_pnu_from_bjd_nm 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 None type)
            - generate_pnu_from_bjd_nm 함수 예외처리 테스트(TypeError 입력값(bjd_cd or jibun)이 문자열이 아닌 bool type)
            - generate_pnu_from_bjd_nm 함수 예외처리 테스트(ValueError 입력값(bjd_cd)이 적절한 형식(10자리 숫자문자열)의 문자열이 아닌 경우들)
            - generate_pnu_from_bjd_nm 함수 예외처리 테스트(ValueError 입력값(jibun)이 적절한 형식의 문자열이 아닌 경우들)

        - generate_pnu_from_bjd_nm 함수 리턴값 테스트(올바른 법정동 문자열 및 지번 제공)
            - generate_pnu_from_bjd_nm 함수 리턴값 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 error 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 pnu 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_cd 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 mountain_cd 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bunji_cd 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_datas 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bun 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 ji 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 msg 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 base_dt 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 error 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 pnu 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_cd 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 mountain_cd 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bunji_cd 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_datas 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bun 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 ji 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 일치 테스트

        - generate_pnu_from_bjd_nm 함수 리턴값 테스트(형식은 맞지만 존재하지 않는 법정동 문자열 및 정상적인 지번 제공)
            - generate_pnu_from_bjd_nm 함수 리턴값 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 error 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 pnu 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_cd 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 mountain_cd 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bunji_cd 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_datas 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bun 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 ji 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 msg 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 base_dt 타입 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 error 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 pnu 값이 None 이 맞는지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_cd 값이 None 이 맞는지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 mountain_cd 값이 None 이 맞는지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bunji_cd 값이 None 이 맞는지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bjd_datas 값이 None 이 맞는지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 bun 값이 None 이 맞는지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 ji 값이 None 이 맞는지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - generate_pnu_from_bjd_nm 함수 리턴값 일치 테스트
        """

        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm=0, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm=0.1, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm=None, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm=True, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm=False, jibun='')
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='', jibun=0)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='', jibun=0.1)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='', jibun=None)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='', jibun=True)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='', jibun=False)

        with self.assertRaises(ValueError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='a 서울특별시 강남구 삼성동', jibun='0000-0000') # 알파벳 포함
        with self.assertRaises(ValueError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='seoulsi gangnamgu samsungdong', jibun='0000-0000') # 알파벳 포함
        with self.assertRaises(ValueError): self.instance.generate_pnu_from_bjd_nm(bjd_nm='ㄱ 서울특별시 강남구 삼성동', jibun='0000-0000') # 불완전한 한글 문자열 포함

        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 00000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 0000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 00000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 000000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 0000000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 11111-2', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 33333-44', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 55555-666', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 77777-8888', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 99999-00000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 7777-88888', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 555-66666', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 33-44444', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 1-22222', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산00000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산0000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산00000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산000000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산0000000000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산11111-2', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산33333-44', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산55555-666', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산77777-8888', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산99999-00000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산7777-88888', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산555-66666', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산33-44444', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산1-22222', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 -1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산  -1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산   -1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산1000-', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 1000-', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산  1000-', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산   1000-', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산 10000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='a', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='a000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='000a', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='a-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='1000-a', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='ㄱ', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='ㄱ000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='000ㄱ', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='ㄱ-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='1000-ㄱ', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='나', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='나000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='000나', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='나-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='1000-나', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='선 1000-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='순 1000-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='상 1000-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='선산 1000-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='나산 1000-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산나 1000-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산산 1000-1000', result=True)
        self.sub_test_generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='산산산 1000-1000', result=True)

        res = self.instance.generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성동', jibun='1-1') # 정상적인 법정동코드 및 지번 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        self.assertIsInstance(res.get('pnu'), str)
        self.assertIsInstance(res.get('bjd_cd'), str)
        self.assertIsInstance(res.get('mountain_cd'), str)
        self.assertIsInstance(res.get('bunji_cd'), str)
        self.assertIsInstance(res.get('bjd_datas'), dict)
        self.assertIsInstance(res.get('bun'), str)
        self.assertIsInstance(res.get('ji'), str)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNotNone(res.get('pnu'))
        self.assertIsNotNone(res.get('bjd_cd'))
        self.assertIsNotNone(res.get('mountain_cd'))
        self.assertIsNotNone(res.get('bunji_cd'))
        self.assertIsNotNone(res.get('bjd_datas'))
        self.assertIsNotNone(res.get('bun'))
        self.assertIsNotNone(res.get('ji'))
        self.assertIsNotNone(res.get('base_dt'))
        self.assertIsNotNone(res.get('msg'))
        self.assertEqual(res, {
            'error': False,
            'pnu': '1168010500100010001',
            'bjd_cd': '1168010500',
            'mountain_cd': '1',
            'bunji_cd': '00010001',
            'bjd_datas': {
                'error': False,
                'sido_nm': '서울특별시',
                'sgg_nm': '강남구',
                'emd_nm': '삼성동',
                'ri_nm': None,
                'full_bjd_nm': '서울특별시 강남구 삼성동',
                'created_dt': '1988-04-23',
                'deleted_dt': None,
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''
            },
            'bun': '1',
            'ji': '1',
            'msg': '',
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
        })

        res = self.instance.generate_pnu_from_bjd_nm(bjd_nm='서울특별시 강남구 삼성1동', jibun='1-1') # 정상적인 법정동코드 및 지번 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        # self.assertIsInstance(res.get('pnu'), str)
        # self.assertIsInstance(res.get('bjd_cd'), str)
        # self.assertIsInstance(res.get('mountain_cd'), str)
        # self.assertIsInstance(res.get('bunji_cd'), str)
        self.assertIsInstance(res.get('bjd_datas'), dict)
        # self.assertIsInstance(res.get('bun'), str)
        # self.assertIsInstance(res.get('ji'), str)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNone(res.get('pnu'))
        self.assertIsNone(res.get('bjd_cd'))
        self.assertIsNone(res.get('mountain_cd'))
        self.assertIsNone(res.get('bunji_cd'))
        self.assertIsNotNone(res.get('bjd_datas'))
        self.assertIsNone(res.get('bun'))
        self.assertIsNone(res.get('ji'))
        self.assertIsNotNone(res.get('msg'))
        self.assertIsNotNone(res.get('base_dt'))
        self.assertEqual(res, {
            'error': True,
            'pnu': None,
            'bjd_cd': None,
            'mountain_cd': None,
            'bunji_cd': None,
            'bjd_datas': {
                'error': True,
                'sido_nm': None,
                'sgg_nm': None,
                'emd_nm': None,
                'ri_nm': None,
                'full_bjd_nm': None,
                'created_dt': None,
                'deleted_dt': None,
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''},
            'bun': None,
            'ji': None,
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': "'서울특별시 강남구 삼성1동' is not a valid legal district name",
        })


    def sub_test_generate_pnu_from_address(
        self,
        address,
        result: bool
    ):
        """
        _generate_pnu 함수 테스트 서브 메소드
            - assertIsInstance() 테스트
            - assertIsNotNone() 테스트
            - assertEqual() 테스트
        """
        res = self.instance.generate_pnu_from_address(address)
        self.assertIsInstance(res.get("error"), bool)
        self.assertIsInstance(res.get("msg"), str)
        self.assertIsNotNone(res)
        self.assertIsNotNone(res.get("error"))
        self.assertIsNotNone(res.get("msg"))
        self.assertEqual(res.get("error"), result)


    def test_generate_pnu_from_address(self):
        """
        generate_pnu_from_address 함수 테스트 메소드

        - generate_pnu_from_address 함수 예외처리 테스트
            - generate_pnu_from_address 함수 예외처리 테스트(TypeError 입력값(address)이 문자열이 아닌 int type)
            - generate_pnu_from_address 함수 예외처리 테스트(TypeError 입력값(address)이 문자열이 아닌 float type)
            - generate_pnu_from_address 함수 예외처리 테스트(TypeError 입력값(address)이 문자열이 아닌 None type)
            - generate_pnu_from_address 함수 예외처리 테스트(TypeError 입력값(address)이 문자열이 아닌 bool type)

        - generate_pnu_from_address 함수 리턴값 테스트(올바른 법정동 문자열 및 지번 제공)
            - generate_pnu_from_address 함수 리턴값 타입 테스트
            - generate_pnu_from_address 함수 리턴값 error 타입 테스트
            - generate_pnu_from_address 함수 리턴값 pnu 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_cd 타입 테스트
            - generate_pnu_from_address 함수 리턴값 mountain_cd 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bunji_cd 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_datas 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bun 타입 테스트
            - generate_pnu_from_address 함수 리턴값 ji 타입 테스트
            - generate_pnu_from_address 함수 리턴값 msg 타입 테스트
            - generate_pnu_from_address 함수 리턴값 base_dt 타입 테스트
            - generate_pnu_from_address 함수 리턴값 error 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 pnu 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_cd 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 mountain_cd 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 bunji_cd 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_datas 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 bun 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 ji 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 일치 테스트

        - generate_pnu_from_address 함수 리턴값 테스트(형식은 맞지만 존재하지 않는 법정동 문자열 및 정상적인 지번 제공)
            - generate_pnu_from_address 함수 리턴값 타입 테스트
            - generate_pnu_from_address 함수 리턴값 error 타입 테스트
            - generate_pnu_from_address 함수 리턴값 pnu 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_cd 타입 테스트
            - generate_pnu_from_address 함수 리턴값 mountain_cd 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bunji_cd 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_datas 타입 테스트
            - generate_pnu_from_address 함수 리턴값 bun 타입 테스트
            - generate_pnu_from_address 함수 리턴값 ji 타입 테스트
            - generate_pnu_from_address 함수 리턴값 msg 타입 테스트
            - generate_pnu_from_address 함수 리턴값 base_dt 타입 테스트
            - generate_pnu_from_address 함수 리턴값 error 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 pnu 값이 None 이 맞는지 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_cd 값이 None 이 맞는지 테스트
            - generate_pnu_from_address 함수 리턴값 mountain_cd 값이 None 이 맞는지 테스트
            - generate_pnu_from_address 함수 리턴값 bunji_cd 값이 None 이 맞는지 테스트
            - generate_pnu_from_address 함수 리턴값 bjd_datas 값이 None 이 맞는지 테스트
            - generate_pnu_from_address 함수 리턴값 bun 값이 None 이 맞는지 테스트
            - generate_pnu_from_address 함수 리턴값 ji 값이 None 이 맞는지 테스트
            - generate_pnu_from_address 함수 리턴값 msg 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 base_dt 값이 None 이 아닌지 테스트
            - generate_pnu_from_address 함수 리턴값 일치 테스트
        """

        with self.assertRaises(TypeError): self.instance.generate_pnu_from_address(address=0)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_address(address=0.1)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_address(address=None)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_address(address=True)
        with self.assertRaises(TypeError): self.instance.generate_pnu_from_address(address=False)

        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 00000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 0000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 00000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 000000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 0000000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 11111-2', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 33333-44', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 55555-666', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 77777-8888', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 99999-00000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 7777-88888', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 555-66666', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 33-44444', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 1-22222', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산00000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산0000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산00000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산000000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산0000000000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산11111-2', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산33333-44', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산55555-666', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산77777-8888', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산99999-00000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산7777-88888', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산555-66666', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산33-44444', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산1-22222', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 -1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산  -1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산   -1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산1000-', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 1000-', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산  1000-', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산   1000-', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산 10000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 a', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 a000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 000a', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 a-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 1000-a', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 ㄱ', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 ㄱ000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 000ㄱ', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 ㄱ-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 1000-ㄱ', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 나', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 나000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 000나', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 나-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 1000-나', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 선 1000-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 순 1000-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 상 1000-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 선산 1000-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 나산 1000-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산나 1000-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산산 1000-1000', result=True)
        self.sub_test_generate_pnu_from_address(address='서울특별시 강남구 삼성동 산산산 1000-1000', result=True)

        res = self.instance.generate_pnu_from_address(address='서울특별시 강남구 삼성동 1-1') # 정상적인 법정동코드 및 지번 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        self.assertIsInstance(res.get('pnu'), str)
        self.assertIsInstance(res.get('bjd_cd'), str)
        self.assertIsInstance(res.get('mountain_cd'), str)
        self.assertIsInstance(res.get('bunji_cd'), str)
        self.assertIsInstance(res.get('bjd_datas'), dict)
        self.assertIsInstance(res.get('bun'), str)
        self.assertIsInstance(res.get('ji'), str)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNotNone(res.get('pnu'))
        self.assertIsNotNone(res.get('bjd_cd'))
        self.assertIsNotNone(res.get('mountain_cd'))
        self.assertIsNotNone(res.get('bunji_cd'))
        self.assertIsNotNone(res.get('bjd_datas'))
        self.assertIsNotNone(res.get('bun'))
        self.assertIsNotNone(res.get('ji'))
        self.assertIsNotNone(res.get('base_dt'))
        self.assertIsNotNone(res.get('msg'))
        self.assertEqual(res, {
            'error': False,
            'pnu': '1168010500100010001',
            'bjd_cd': '1168010500',
            'mountain_cd': '1',
            'bunji_cd': '00010001',
            'bjd_datas': {
                'error': False,
                'sido_nm': '서울특별시',
                'sgg_nm': '강남구',
                'emd_nm': '삼성동',
                'ri_nm': None,
                'full_bjd_nm': '서울특별시 강남구 삼성동',
                'created_dt': '1988-04-23',
                'deleted_dt': None,
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''
            },
            'bun': '1',
            'ji': '1',
            'msg': '',
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
        })

        res = self.instance.generate_pnu_from_address(address='서울특별시 강남구 삼성1동 1-1') # 정상적인 법정동코드 및 지번 입력값
        self.assertIsInstance(res, dict)
        self.assertIsInstance(res.get('error'), bool)
        # self.assertIsInstance(res.get('pnu'), str)
        # self.assertIsInstance(res.get('bjd_cd'), str)
        # self.assertIsInstance(res.get('mountain_cd'), str)
        # self.assertIsInstance(res.get('bunji_cd'), str)
        self.assertIsInstance(res.get('bjd_datas'), dict)
        # self.assertIsInstance(res.get('bun'), str)
        # self.assertIsInstance(res.get('ji'), str)
        self.assertIsInstance(res.get('msg'), str)
        self.assertIsInstance(res.get('base_dt'), str)
        self.assertIsNotNone(res.get('error'))
        self.assertIsNone(res.get('pnu'))
        self.assertIsNone(res.get('bjd_cd'))
        self.assertIsNone(res.get('mountain_cd'))
        self.assertIsNone(res.get('bunji_cd'))
        self.assertIsNotNone(res.get('bjd_datas'))
        self.assertIsNone(res.get('bun'))
        self.assertIsNone(res.get('ji'))
        self.assertIsNotNone(res.get('msg'))
        self.assertIsNotNone(res.get('base_dt'))
        self.assertEqual(res, {
            'error': True,
            'pnu': None,
            'bjd_cd': None,
            'mountain_cd': None,
            'bunji_cd': None,
            'bjd_datas': {
                'error': True,
                'sido_nm': None,
                'sgg_nm': None,
                'emd_nm': None,
                'ri_nm': None,
                'full_bjd_nm': None,
                'created_dt': None,
                'deleted_dt': None,
                'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
                'msg': ''},
            'bun': None,
            'ji': None,
            'base_dt': self.instance.base_dt_print, # base_dt_print 는 업데이트 되므로 self.instance.base_dt_print 로 적용
            'msg': 'Failed to extract bjd name from address: 서울특별시 강남구 삼성1동 1-1',
        })


if __name__ == "__main__":
    unittest.main()